
BEGIN {
  unless ($ENV{RELEASE_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for release candidate testing');
  }
}

        use strict;
        use warnings;

        use Test::More;

        ## no critic
        eval 'use Test::Code::TidyAll 0.20';
        plan skip_all =>
            "Test::Code::TidyAll 0.20 required to check if the code is clean."
            if $@;
        tidyall_ok();
